package cn.zgfxrc.boot.upms.biz.controller;

import cn.hutool.core.io.FileUtil;
import cn.zgfxrc.boot.upms.api.vo.FileInfoVo;
import cn.zgfxrc.boot.upms.biz.service.FileTemplate;
import com.seepine.secret.annotation.Expose;
import com.seepine.secret.annotation.Log;
import com.seepine.tool.secure.symmetric.Base64;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.tasks.UnsupportedFormatException;
import org.jetbrains.annotations.Nullable;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@Slf4j
@RestController
@Api(tags = "文件管理")
@RequestMapping("sys/file")
@AllArgsConstructor
public class SysFileController {
  private final FileTemplate fileTemplate;

  @ApiOperation("上传文件")
  @Log(module = "upms", title = "上传文件")
  @PostMapping("upload")
  public FileInfoVo upload(@RequestParam("file") @Nullable MultipartFile file)
      throws WrapException {
    if (file == null || file.isEmpty() || !StringUtils.hasText(file.getOriginalFilename())) {
      throw new WrapException("找不到上传的文件");
    }
    return fileTemplate.upload(file);
  }

  /**
   * 上传图片接口返回png，支持图片压缩，例如压缩为宽高最大值不超过80，压缩率80%，返回base64值：/sys/file/upload/image?size=80&quality=0.8&base64=1
   *
   * @param file 图片文件
   * @param size 最大宽高尺寸
   * @param width 最大宽度
   * @param height 最大高度
   * @param scale 缩放比例，1.0为100%
   * @param rotate 旋转角度
   * @param quality 压缩质量
   * @param base64 1表示返回的url为base64图片数据
   * @return {url,filename,originalFilename}
   * @throws WrapException 异常
   */
  @ApiOperation("上传图片")
  @Log(module = "upms", title = "上传文件")
  @PostMapping("upload/image")
  public FileInfoVo uploadImg(
      @RequestParam("file") @Nullable MultipartFile file,
      Integer size,
      Integer width,
      Integer height,
      Double scale,
      Integer rotate,
      Double quality,
      Integer base64)
      throws WrapException {
    if (file == null
        || file.isEmpty()
        || file.getSize() <= 0
        || !StringUtils.hasText(file.getOriginalFilename())) {
      throw new WrapException("找不到上传的图片");
    }
    String fileType = FileUtil.getMimeType(file.getOriginalFilename());
    String fileExt = fileType == null ? "png" : fileType.replace("image/", "");
    ByteArrayOutputStream os = new ByteArrayOutputStream();
    try {
      Thumbnails.Builder<?> builder = Thumbnails.of(file.getInputStream());
      if (width == null) {
        width = 2000;
      }
      if (height == null) {
        height = 2000;
      }
      // 尺寸缩放
      if (size != null) {
        builder.size(size, size);
      } else if (scale != null) {
        builder.scale(scale);
      } else {
        builder.size(width, height);
      }
      if (rotate != null) {
        builder.rotate(rotate);
      }
      // 50KB以内不压缩
      builder.outputQuality(
          quality != null ? quality : (file.getSize() > 1024 * 50 ? 0.85f : 1.0f));
      builder.imageType(BufferedImage.TYPE_INT_ARGB).outputFormat(fileExt).toOutputStream(os);
    } catch (UnsupportedFormatException e) {
      throw new WrapException("不支持此图片格式");
    } catch (IOException e) {
      throw new WrapException("上传失败[IO错误]");
    } catch (IllegalArgumentException e) {
      throw new WrapException(e.getMessage());
    }
    if (Integer.valueOf("1").equals(base64)) {
      FileInfoVo vo = new FileInfoVo();
      vo.setExt(fileExt);
      vo.setFilename(fileTemplate.getFileName(file.getOriginalFilename(), fileExt));
      vo.setOriginalFilename(file.getOriginalFilename());
      vo.setUrl("data:image/png;base64," + Base64.encode(os.toByteArray()));
      return vo;
    }
    return fileTemplate.upload(fileTemplate.getFileName(file.getOriginalFilename(), fileExt), os);
  }

  @Expose
  @GetMapping("get/{filename}")
  public void get(@PathVariable String filename, HttpServletResponse response) throws IOException {
    fileTemplate.download(filename, response);
  }

  @PostMapping("del/{fileName}")
  public void del(@PathVariable String fileName) {
    fileTemplate.delete(fileName);
  }
}
