package cn.zgfxrc.boot.upms.biz.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileTypeUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.io.IoUtil;
import cn.xuyanwu.spring.file.storage.FileInfo;
import cn.xuyanwu.spring.file.storage.FileStorageService;
import cn.xuyanwu.spring.file.storage.exception.FileStorageRuntimeException;
import cn.zgfxrc.boot.upms.api.vo.FileInfoVo;
import com.github.yitter.idgen.YitIdHelper;
import com.seepine.json.JsonObject;
import com.seepine.wrap.WrapException;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

/**
 * @author huanghs
 */
@Slf4j
@Service
@AllArgsConstructor
public class FileTemplate {
  private final SysFileService sysFileService;
  private final FileStorageService fileStorageService;

  @NotNull
  public FileInfoVo upload(@NotNull MultipartFile file) {
    ByteArrayOutputStream os = new ByteArrayOutputStream();
    try {
      InputStream is = file.getInputStream();
      IoUtil.copy(is, os);
    } catch (IOException e) {
      throw new WrapException("读取文件错误，请重新上传");
    }
    return upload(getFileName(os, file.getOriginalFilename()), os);
  }

  public String getType(ByteArrayOutputStream os, String originName) {
    String type = FileTypeUtil.getType(new ByteArrayInputStream(os.toByteArray()), originName);
    if (!StringUtils.hasText(type)) {
      // 次选从文件名判断类型
      type = FileUtil.extName(originName);
      if (!StringUtils.hasText(type)) {
        log.warn("获取OriginalFilename: {} ,文件扩展名失败，主动改为jpg", originName);
        log.warn("其originName: {}", originName);
        // 都拿不到文件类型，默认当成图片处理
        type = "jpeg";
      }
    }
    return type;
  }

  public String getFileName(ByteArrayOutputStream os, String originalFilename) {
    String fileName =
        StringUtils.hasText(originalFilename)
            ? originalFilename
            : String.valueOf(YitIdHelper.nextId());
    return getFileName(fileName, getType(os, fileName));
  }

  public String getFileName(String fileName, String type) {
    if (StringUtils.hasText(fileName) && fileName.endsWith(type)) {
      fileName = fileName.substring(0, fileName.length() - type.length() - 1);
    }
    return fileName + "." + type.toLowerCase();
  }

  @NotNull
  public FileInfoVo upload(
      @NotNull String fileName, @NotNull ByteArrayOutputStream byteArrayOutputStream) {
    String saveFilename = YitIdHelper.nextId() + "." + FileUtil.extName(fileName);
    FileInfo fileInfo =
        fileStorageService
            .of(new ByteArrayInputStream(byteArrayOutputStream.toByteArray()))
            .setOriginalFilename(fileName)
            .setSaveFilename(saveFilename)
            .setSaveThFilename(saveFilename)
            .upload();
    return BeanUtil.copyProperties(fileInfo, FileInfoVo.class);
  }
  /**
   * 上传base64
   *
   * @param fileName 文件名
   * @param base64 base64
   * @return FileInfo
   */
  @NotNull
  public FileInfoVo upload(@NotNull String fileName, @NotNull String base64) {
    ByteArrayOutputStream os = new ByteArrayOutputStream();
    Base64.decodeToStream(base64, os, false);
    return upload(fileName, os);
  }

  /**
   * 通过文件名获取文件信息
   *
   * @param filename OSS文件名
   * @return 文件信息
   */
  public FileInfoVo get(String filename) {
    return BeanUtil.copyProperties(sysFileService.getByFilename(filename), FileInfoVo.class);
  }

  /**
   * 根据文件名下载文件
   *
   * @param filename OSS文件名
   * @param response res
   */
  public void download(String filename, HttpServletResponse response) throws IOException {
    String type = FileUtil.getMimeType(filename);
    String view = null;
    if (StringUtils.hasText(type)) {
      response.setContentType(type + ";charset=utf-8");
      if (type.startsWith("image")
          || type.startsWith("text")
          || type.contains("pdf")
          || type.contains("javascript")) {
        view = "1";
      }
    }
    response.setHeader("Cache-Control", "max-age=604800");
    response.setHeader(
        "Content-Disposition",
        (view != null ? "inline" : "attachment")
            + ";filename="
            + URLEncoder.encode(filename, StandardCharsets.UTF_8));
    try {
      download(filename, response.getOutputStream());
    } catch (Exception e) {
      response.setHeader("Cache-Control", "no-store");
      response.setContentType("application/json;charset=utf-8");
      ServletOutputStream out = response.getOutputStream();
      OutputStreamWriter ow = new OutputStreamWriter(out, StandardCharsets.UTF_8);
      if (e instanceof FileStorageRuntimeException) {
        response.setStatus(404);
        ow.write(JsonObject.build().set("code", 404).set("msg", "文件不存在").toJson());
      } else if (e instanceof IOException || e instanceof IORuntimeException) {
        response.setStatus(500);
        ow.write(JsonObject.build().set("code", 500).set("msg", "下载文件失败").toJson());
      } else {
        response.setStatus(500);
        ow.write(JsonObject.build().set("code", 500).set("msg", e.getMessage()).toJson());
      }
      ow.flush();
      ow.close();
    }
  }
  /**
   * 根据文件名下载文件
   *
   * @param filename OSS文件名
   * @param outputStream 输出流
   */
  public void download(String filename, OutputStream outputStream) {
    fileStorageService.download(sysFileService.getByFilename(filename)).outputStream(outputStream);
  }

  /**
   * 删除文件，并从数据库删除
   *
   * @param filename OSS文件名
   */
  public void delete(String filename) {
    sysFileService.delete(filename);
  }
}
