package cn.zgfxrc.boot.upms.biz.service;

import cn.zgfxrc.boot.common.core.entity.base.BaseModel;
import cn.zgfxrc.boot.common.core.enums.MenuType;
import cn.zgfxrc.boot.common.core.enums.PlatformType;
import cn.zgfxrc.boot.common.core.util.PageUtil;
import cn.zgfxrc.boot.common.core.util.TreeUtil;
import cn.zgfxrc.boot.upms.api.entity.SysMenu;
import cn.zgfxrc.boot.upms.api.entity.SysRole;
import cn.zgfxrc.boot.upms.api.entity.SysRoleMenu;
import cn.zgfxrc.boot.upms.api.entity.res.SysMenuRes;
import cn.zgfxrc.boot.upms.api.entity.res.SysMenuTreeNode;
import cn.zgfxrc.boot.upms.biz.mapper.SysMenuMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysPackageMenuMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysRoleMenuMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.seepine.secret.AuthUtil;
import com.seepine.secret.entity.AuthUser;
import com.seepine.tool.cache.Cache;
import com.seepine.tool.secure.digest.DigestAlgorithm;
import com.seepine.tool.secure.digest.Digests;
import com.seepine.tool.util.ListUtil;
import com.seepine.tool.util.Objects;
import com.seepine.tool.util.Validate;
import java.io.Serializable;
import java.time.Duration;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Nullable;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysMenuService extends ServiceImpl<SysMenuMapper, SysMenu> {
  private final SysRoleService sysRoleService;
  private final SysRoleMenuMapper sysRoleMenuMapper;
  private final SysPackageMenuMapper sysPackageMenuMapper;
  private final SysPackageService sysPackageService;
  private static final String CACHE_PREFIX = "sys_menu";

  @Override
  public SysMenu getById(Serializable id) {
    return Cache.getIfPresent(
        CACHE_PREFIX + ":id:" + id, () -> super.getById(id), Duration.ofDays(1).toMillis());
  }

  @Override
  public boolean save(SysMenu entity) {
    entity.setParentId(entity.getParentId() == null ? 0L : entity.getParentId());
    Cache.removeByPattern(CACHE_PREFIX + ":global*");
    return super.save(entity);
  }

  @Override
  public boolean updateById(SysMenu entity) {
    Cache.removeByPattern(CACHE_PREFIX + ":global*");
    Cache.remove(CACHE_PREFIX + ":id:" + entity.getId());
    return super.updateById(entity);
  }

  @Override
  public boolean removeById(Serializable id) {
    Cache.removeByPattern(CACHE_PREFIX + ":global*");
    Cache.remove(CACHE_PREFIX + ":id:" + id);
    return super.removeById(id);
  }

  public void removeGlobalCache() {
    Cache.removeByPattern(CACHE_PREFIX + ":global*");
  }

  public IPage<SysMenuRes> getPageRes(Page<SysMenu> page, SysMenu entity) {
    LambdaQueryWrapper<SysMenu> wrapper =
        Wrappers.lambdaQuery(entity).isNull(SysMenu::getParentId).orderByAsc(SysMenu::getSort);
    return PageUtil.convertPage(
        super.page(page, wrapper),
        item -> {
          SysMenuRes res = new SysMenuRes();
          BeanUtils.copyProperties(item, res);
          res.setChildren(this.findAllChildren(item.getId()));
          return res;
        });
  }

  /**
   * 获取菜单树结构
   *
   * @param entity req
   * @return list
   * @date 2022.11.28
   */
  public List<SysMenuTreeNode> fetch(SysMenu entity) {
    return TreeUtil.build(
        lambdaQuery()
            .orderByAsc(SysMenu::getSort)
            .eq(SysMenu::getPlatformType, entity.getPlatformType())
            .list()
            .stream()
            .map(
                item -> {
                  SysMenuTreeNode node = new SysMenuTreeNode();
                  BeanUtils.copyProperties(item, node);
                  return node;
                })
            .collect(Collectors.toList()));
  }

  /**
   * 获取菜单树结构，无返回icon
   *
   * @param entity req
   * @return list
   * @date 2022.11.28
   */
  public List<SysMenuTreeNode> dicTree(SysMenu entity) {
    return TreeUtil.build(
        lambdaQuery()
            .orderByAsc(SysMenu::getSort)
            .eq(SysMenu::getPlatformType, entity.getPlatformType())
            .list()
            .stream()
            .map(
                item -> {
                  SysMenuTreeNode node = new SysMenuTreeNode();
                  BeanUtils.copyProperties(item, node);
                  node.setIcon(null);
                  return node;
                })
            .collect(Collectors.toList()));
  }

  /**
   * 获取当前登录用户拥有的菜单树
   *
   * @return list(tree)
   */
  public List<SysMenuTreeNode> tree() {
    AuthUser user = AuthUtil.getUser();
    PlatformType platformType = PlatformType.from(user.getClaim("platformType"));
    LambdaQueryChainWrapper<SysMenu> wrapper =
        super.lambdaQuery().eq(SysMenu::getPlatformType, platformType).orderByAsc(SysMenu::getSort);

    UserMenu userMenu = calUserMenu(user);
    // 非超管或非平台，都需要过滤
    if (userMenu.getNeedFilter()) {
      if (Objects.isEmpty(userMenu.getMenuIds())) {
        return new ArrayList<>();
      }
      wrapper.in(BaseModel::getId, userMenu.getMenuIds());
    }

    // 获取所拥有的所有菜单和按钮
    List<SysMenu> list = wrapper.list();
    // 递归获取父级菜单
    List<SysMenu> allList =
        ListUtil.filter(getAllMenus(list), (menu) -> menu.getMenuType().equals(MenuType.MENU));
    return TreeUtil.build(
        allList.stream()
            .map(
                item -> {
                  SysMenuTreeNode node = new SysMenuTreeNode();
                  BeanUtils.copyProperties(item, node);
                  return node;
                })
            .toList());
  }

  /**
   * 根据角色ids获取菜单ids
   *
   * @param roleIds 角色ids
   * @return menuIds
   */
  public List<Long> getMenuIds(List<Long> roleIds) {
    if (Objects.isEmpty(roleIds)) {
      return new ArrayList<>();
    }
    return ListUtil.map(
        sysRoleMenuMapper.selectList(
            Wrappers.<SysRoleMenu>lambdaQuery().in(SysRoleMenu::getRoleId, roleIds)),
        SysRoleMenu::getMenuId);
  }

  /**
   * 获取权限列表
   *
   * @return String[]
   * @date 2021.11.8 huanghs@zgfxrc.cn
   */
  public List<String> getPermissions(AuthUser user) {
    String cacheKey = ":list";
    LambdaQueryChainWrapper<SysMenu> wrapper =
        super.lambdaQuery().isNotNull(SysMenu::getPermission).ne(SysMenu::getPermission, "");
    UserMenu userMenu = calUserMenu(user);
    // 非超管或非平台，都需要过滤
    if (userMenu.getNeedFilter()) {
      if (Objects.isEmpty(userMenu.getMenuIds())) {
        return new ArrayList<>();
      }
      wrapper.in(BaseModel::getId, userMenu.getMenuIds());
      cacheKey +=
          Digests.digest(DigestAlgorithm.MD5).digestBase64(userMenu.getMenuIds().toString());
    }
    return ListUtil.map(
        ListUtil.filter(
            // 搜索父级
            getAllMenus(
                Cache.getIfPresent(
                    CACHE_PREFIX + ":global:get_not_permission" + cacheKey,
                    wrapper::list,
                    Duration.ofDays(1).toMillis())),
            menu -> Objects.nonBlank(menu.getPermission())),
        SysMenu::getPermission);
  }

  /**
   * 通过父id获取所有子树
   *
   * @param parentId 父id
   * @return list
   */
  public List<SysMenuRes> findAllChildren(Long parentId) {
    List<SysMenuRes> list =
        ListUtil.map(
            super.lambdaQuery()
                .eq(SysMenu::getParentId, parentId)
                .orderByAsc(SysMenu::getSort)
                .list(),
            item -> {
              SysMenuRes res = new SysMenuRes();
              BeanUtils.copyProperties(item, res);
              res.setChildren(findAllChildren(item.getId()));
              return res;
            });
    return Objects.isEmpty(list) ? null : list;
  }

  /**
   * 根据角色id获取菜单权限，用于菜单权限配置
   *
   * @param roleId 角色id
   * @return {checkedIds:[],tree:[]}
   */
  @SneakyThrows
  public SysMenuRes treeByRoleId(Long roleId) {
    SysRole role = sysRoleService.getById(roleId);
    Validate.nonEmpty(role, "找不到角色信息，请刷新页面后重试");
    SysMenuRes res = new SysMenuRes();
    List<Long> checkedMenuIds = this.getMenuIds(Collections.singletonList(roleId));
    AuthUser user = AuthUtil.getUser();
    LambdaQueryChainWrapper<SysMenu> wrapper = super.lambdaQuery();
    // 只能拿套餐内的
    if (!PlatformType.SYS.eq(user.getClaim("platformType"))) {
      // 过滤为当前租户所拥有菜单
      List<Long> haveMenuIds = sysPackageService.getMenuIdsByTenantId(user.getTenantIdAsLong());
      wrapper.in(BaseModel::getId, haveMenuIds);
      if (haveMenuIds.isEmpty()) {
        res.setTree(new ArrayList<>());
        res.setCheckedIds(new ArrayList<>());
        return res;
      }
    }
    // 获取所拥有的所有菜单
    List<SysMenu> list =
        getAllMenus(
            wrapper
                .eq(SysMenu::getPlatformType, PlatformType.from(user.getClaim("platformType")))
                .orderByAsc(SysMenu::getSort)
                .list());
    res.setTree(
        TreeUtil.build(
            list.parallelStream()
                .map(
                    item -> {
                      SysMenuTreeNode node = new SysMenuTreeNode();
                      BeanUtils.copyProperties(item, node);
                      return node;
                    })
                .toList()));
    res.setCheckedIds(checkedMenuIds);
    return res;
  }

  /**
   * 根据套餐id获取菜单权限，用于菜单权限配置
   *
   * @param packageId 套餐id
   * @return {checkedIds:[],tree:[]}
   */
  @SneakyThrows
  public SysMenuRes treeByPackageId(Long packageId) {
    SysMenuRes res = new SysMenuRes();
    List<Long> checkedMenuIds = sysPackageMenuMapper.getMenuIds(packageId);
    List<SysMenuTreeNode> tree = new ArrayList<>();
    tree.add(getPlatformTree(PlatformType.ENTERPRISE));
    res.setTree(tree);
    res.setCheckedIds(checkedMenuIds);
    return res;
  }

  /**
   * 创建默认树节点(如企业端)
   *
   * @param platformType 平台类型
   * @return 节点
   */
  private SysMenuTreeNode getPlatformTree(PlatformType platformType) {
    List<SysMenu> partnerList =
        lambdaQuery()
            .orderByAsc(SysMenu::getMenuType)
            .eq(SysMenu::getPlatformType, platformType)
            .orderByAsc(SysMenu::getSort)
            .list();
    SysMenuTreeNode partnerNode = new SysMenuTreeNode();
    partnerNode.setId(platformType.id);
    partnerNode.setName(platformType.label);
    partnerNode.setDisableCheckbox(true);
    partnerNode.setChildren(
        TreeUtil.build(
            partnerList.stream()
                .map(
                    item -> {
                      SysMenuTreeNode node = new SysMenuTreeNode();
                      BeanUtils.copyProperties(item, node);
                      return node;
                    })
                .toList()));
    return partnerNode;
  }

  /**
   * 根据菜单list获取所有关联菜单，例如传进来[1.1.2],需要找寻所有关联菜单为[1, 1.1, 1.1.2]
   *
   * @param menuIds menus
   * @return all menus
   */
  public List<SysMenu> getAllMenus(@Nullable List<SysMenu> menuIds) {
    return Cache.getIfPresent(
        CACHE_PREFIX
            + ":global:get_all_menus:"
            + Digests.digest(DigestAlgorithm.MD5).digestBase64(Objects.toString(menuIds)),
        () -> {
          List<SysMenu> news = new ArrayList<>();
          if (Objects.isEmpty(menuIds)) {
            return news;
          }
          Set<Long> ids = new HashSet<>();
          for (SysMenu menu : menuIds) {
            if (!ids.contains(menu.getId())) {
              news.add(menu);
              ids.add(menu.getId());
            }
            // 判断是否有父节点
            if (menu.getParentId() != null && menu.getParentId() > 0) {
              // 如果父节点已找过，不需要重复找
              if (!ids.contains(menu.getParentId())) {
                SysMenu parent = getById(menu.getParentId());
                if (parent != null) {
                  List<SysMenu> relation = getAllMenus(Collections.singletonList(parent));
                  for (SysMenu sysMenu : relation) {
                    if (!ids.contains(sysMenu.getId())) {
                      news.add(sysMenu);
                      ids.add(sysMenu.getId());
                    }
                  }
                }
              }
            }
          }
          return news.stream()
              .sorted(Comparator.comparing(SysMenu::getSort))
              .collect(Collectors.toList());
        },
        Duration.ofDays(1).toMillis());
  }

  /**
   * 计算用户是否需要过滤菜单ids，以及返回菜单ids数据
   *
   * @param user AuthUser
   * @return UserMenu
   */
  public UserMenu calUserMenu(AuthUser user) {
    boolean isSysPlatform = PlatformType.SYS.eq(user.getClaim("platformType"));
    List<Long> menuIds = new ArrayList<>();
    boolean isSystem = "true".equals(user.getClaim("isSystem"));
    // 是超管
    if (isSystem) {
      // 如果非平台，需要过滤为套餐拥有的权限
      if (!isSysPlatform) {
        menuIds = sysPackageService.getMenuIdsByTenantId(user.getTenantIdAsLong());
      }
    }
    // 非超管
    else {
      // 根据用户拥有的角色，获取menuIds
      menuIds = getMenuIds(sysRoleService.getRoleIds(user.getIdAsLong()));
      // 如果非平台，且角色拥有菜单不为空，还需要根据套餐过滤
      if (!isSysPlatform && !menuIds.isEmpty()) {
        // 过滤为当前租户所拥有菜单
        List<Long> haveMenuIds = sysPackageService.getMenuIdsByTenantId(user.getTenantIdAsLong());
        menuIds = ListUtil.filter(menuIds, haveMenuIds::contains);
      }
    }
    return UserMenu.builder()
        // 非超管 或非平台端 需要过滤菜单ids
        // 即 超管且平台端才不需要，其他都需要
        .needFilter(!isSystem || !isSysPlatform)
        .menuIds(menuIds)
        .build();
  }

  @Builder
  @Getter
  static class UserMenu {
    Boolean needFilter;
    List<Long> menuIds;
  }
}
