package cn.zgfxrc.boot.upms.biz.service;

import cn.zgfxrc.boot.common.core.enums.PlatformType;
import cn.zgfxrc.boot.upms.api.entity.SysRole;
import cn.zgfxrc.boot.upms.api.entity.SysRoleMenu;
import cn.zgfxrc.boot.upms.api.entity.SysUserRole;
import cn.zgfxrc.boot.upms.biz.mapper.SysRoleMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysRoleMenuMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysUserRoleMapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.seepine.tool.util.ListUtil;
import com.seepine.tool.util.Objects;
import com.seepine.tool.util.Validate;
import com.seepine.wrap.WrapException;
import java.io.Serializable;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysRoleService extends ServiceImpl<SysRoleMapper, SysRole> {
  private final SysUserRoleMapper sysUserRoleMapper;
  private final SysRoleMenuMapper sysRoleMenuMapper;

  /**
   * 创建角色
   *
   * @param roleName 角色名
   * @param platformType 平台类型
   * @date 2022.12.28
   */
  @Transactional(rollbackFor = Exception.class)
  public void createRole(String roleName, PlatformType platformType) {
    SysRole role = new SysRole();
    role.setName(roleName);
    role.setPlatformType(platformType);
    Validate.isTrue(super.save(role), "创建[" + roleName + "]角色失败");
  }

  @SneakyThrows
  @Override
  public boolean removeById(Serializable id) {
    return super.removeById(id);
  }

  /**
   * 根据用户id获取角色ids
   *
   * @param userId 用户ids
   * @return roleIds
   */
  public List<Long> getRoleIds(Long userId) {
    return ListUtil.map(
        sysUserRoleMapper.selectList(
            Wrappers.<SysUserRole>lambdaQuery().eq(SysUserRole::getUserId, userId)),
        SysUserRole::getRoleId);
  }

  /**
   * 生成角色菜单关联
   *
   * @param roleId 角色id
   * @param menuIds 菜单ids
   */
  @SneakyThrows
  public void saveRoleMenus(Long roleId, List<Long> menuIds) {
    if (Objects.isEmpty(menuIds)) {
      return;
    }
    Set<Long> set = new HashSet<>(menuIds);
    List<SysRoleMenu> list =
        set.stream()
            .map(
                menuId -> {
                  SysRoleMenu roleMenu = new SysRoleMenu();
                  roleMenu.setRoleId(roleId);
                  roleMenu.setMenuId(menuId);
                  return roleMenu;
                })
            .collect(Collectors.toList());
    if (!Objects.isEmpty(list)) {
      if (sysRoleMenuMapper.insertBatchSomeColumn(list) <= 0) {
        throw new WrapException("保存失败");
      }
    }
  }
  /**
   * 重置角色菜单数据
   *
   * @param roleId 角色id
   * @param menuIds 菜单ids
   * @date 2021.11.8 huanghs@zgfxrc.cn
   */
  @Transactional(rollbackFor = Exception.class)
  public void resetRoleMenus(Long roleId, List<Long> menuIds) {
    removeRoleMenus(roleId);
    saveRoleMenus(roleId, menuIds);
  }

  /**
   * 移除角色所有菜单
   *
   * @param roleId 角色id
   * @date 2021.11.8 huanghs@zgfxrc.cn
   */
  public void removeRoleMenus(Long roleId) {
    sysRoleMenuMapper.delete(
        Wrappers.<SysRoleMenu>lambdaQuery().eq(SysRoleMenu::getRoleId, roleId));
  }

  /**
   * 通过用户id和角色ids重新建立关联，会移除之前的关联
   *
   * @param userId 用户id
   * @param roleIds 角色ids
   * @throws WrapException exception
   * @date 2021.11.8 huanghs@zgfxrc.cn
   */
  @Transactional(rollbackFor = Exception.class)
  public void resetUserRoles(Long userId, List<Long> roleIds) throws WrapException {
    removeRolesByUserId(userId);
    saveUserRoles(userId, roleIds);
  }

  /**
   * 保存用户角色关联
   *
   * @param userId 用户id
   * @param roleIds 角色ids
   * @throws WrapException wrap
   * @date 2021.11.15 huanghs@zgfxrc.cn
   */
  public void saveUserRoles(Long userId, List<Long> roleIds) throws WrapException {
    List<SysUserRole> list =
        ListUtil.map(
            roleIds,
            roleId -> {
              SysUserRole roleMenu = new SysUserRole();
              roleMenu.setRoleId(roleId);
              roleMenu.setUserId(userId);
              return roleMenu;
            });
    if (Objects.nonEmpty(list)) {
      if (sysUserRoleMapper.insertBatchSomeColumn(list) <= 0) {
        throw new WrapException("保存角色权限失败");
      }
    }
  }

  /**
   * 通过用户id删除关联的角色
   *
   * @param userId 用户id
   * @date 2021.11.8 huanghs@zgfxrc.cn
   */
  public void removeRolesByUserId(Long userId) {
    sysUserRoleMapper.delete(
        Wrappers.<SysUserRole>lambdaQuery().eq(SysUserRole::getUserId, userId));
  }
}
